/*****************************************************************************
 *                                                                           *
 * File:                                                                     *
 *  offload.h                                                                *
 *                                                                           *
 * Description:                                                              *
 *  TCP offload support.                                                     *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License, version 2, as       *
 * published by the Free Software Foundation.                                *
 *                                                                           *
 * You should have received a copy of the GNU General Public License along   *
 * with this program; if not, write to the Free Software Foundation, Inc.,   *
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.                 *
 *                                                                           *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED    *
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF      *
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.                     *
 *                                                                           *
 * http://www.chelsio.com                                                    *
 *                                                                           *
 * Copyright (c) 2003 - 2005 Chelsio Communications, Inc.                    *
 * All rights reserved.                                                      *
 *                                                                           *
 * Maintainers: maintainers@chelsio.com                                      *
 *                                                                           *
 * Authors: Dimitrios Michailidis   <dm@chelsio.com>                         *
 *                                                                           *
 * History:                                                                  *
 *                                                                           *
 ****************************************************************************/
/* $Date: 2005/08/16 05:38:32 $ $RCSfile: offload.h,v $ $Revision: 1.16 $ */

#ifndef _NET_OFFLOAD_H
#define _NET_OFFLOAD_H

#include <linux/skbuff.h>
#include <net/sock.h>
#include <linux/toedev.h>

enum {
	OFFLOAD_LISTEN_START,
	OFFLOAD_LISTEN_STOP
};

extern struct proto tcp_prot;

/* Returns true if sk is an offloaded IPv4 TCP socket. */
#define is_offloaded(sk) (((sk)->sk_family == AF_INET && (sk)->sk_prot != &tcp_prot))
/*static inline int is_offloaded(struct sock *sk)
{
	return ((sk)->sk_family == AF_INET && (sk)->sk_prot != &tcp_prot);
}*/

/* Per-skb backlog handler.  Run when a socket's backlog is processed. */
struct blog_skb_cb {
	void (*backlog_rcv)(struct sock *sk, struct sk_buff *skb);
	struct toedev *dev;
};

#define BLOG_SKB_CB(skb) ((struct blog_skb_cb *)(skb)->cb)

/* belongs in linux/tcp_diag.h */
#define TCPDIAG_OFFLOAD 5

/* so does this */
struct tcpdiag_offload {
	unsigned int offload_dev_idx;
	unsigned int offload_cookie;
	unsigned int mem;
};

struct notifier_block;

/*
 * TCP operations that a TOE wants to override but cannot through existing
 * means.
 */
struct toe_funcs {
	void (*rcv_consumed)(struct sock *sk, int consumed);
	void (*pmtu_changed)(struct sock *sk);
	void (*set_keepalive)(struct sock *sk, int on_off);
	void (*tcpdiag_offload_info)(const struct sock *sk,
				     struct tcpdiag_offload *oinfo);
	int  (*sendskb)(struct sock *sk, struct sk_buff *skb, int flags);
};

extern int register_listen_offload_notifier(struct notifier_block *nb);
extern int unregister_listen_offload_notifier(struct notifier_block *nb);
extern int tcp_listen_offload_stop(struct sock *sk);
extern int tcp_listen_offload(struct sock *sk);
extern int tcp_connect_offload(struct sock *sk);
#endif /* _NET_OFFLOAD_H */
